package org.ibex.crypto;
import java.security.SecureRandom;

public class PKCS1 {
    private final RSA rsa;
    private final SecureRandom srand;
    public PKCS1(RSA rsa) { this(rsa,new SecureRandom()); }
    public PKCS1(RSA rsa,SecureRandom srand) { this.rsa = rsa; this.srand = srand; }
    
    public byte[] encode(byte[] in) {
        int size = rsa.getInputBlockSize();
        if(in.length > size -  11) throw new IllegalArgumentException("message too long");
        byte[] buf = new byte[size];
        byte[] rand = new byte[size - in.length - 2];
        srand.nextBytes(rand);
        for(int i=0;i<rand.length;i++) while(rand[i] == 0) rand[i] = (byte)srand.nextInt();
        int p=0;
        buf[p++] = 0x02;
        System.arraycopy(rand,0,buf,p,rand.length);
        p+=rand.length;
        buf[p++]  = 0x0;
        System.arraycopy(in,0,buf,p,in.length);

        return rsa.process(buf);
    }
    
    public byte[] decode(byte[] in) throws Exn {
        byte[] buf = rsa.process(in);
        if(buf.length < 10) throw new Exn("Data too short");
        if(buf[0] != 2 && buf[0] != 1) throw new Exn("Data not in correct format " + (buf[0]&0xff));
        int start = 9;
        while(start < buf.length && buf[start] != 0) start++;
        if(start == buf.length) throw new Exn("No null separator");
        start++;
        byte[] ret = new byte[buf.length - start];
        System.arraycopy(buf,start,ret,0,ret.length);
        return ret;
    }
    
    public static class Exn extends Exception { public Exn(String s) { super(s); } }
}
